@@ -1,4 +1,4 @@
-# $Id: Best.pm,v 1.9 2005/01/14 06:08:35 reid Exp $
+# $Id$
 
 #   Algorithm::Pair::Best.pm
 #
@@ -13,74 +13,10 @@
 #   or FITNESS FOR A PARTICULAR PURPOSE.
 #
 
-=head1 NAME
-
-Algorithm::Pair::Best - Perl module to select pairings (designed for Go
-tournaments, but can be used for anything, really).
-
-=head1 SYNOPSIS
-
-    use Algorithm::Pair::Best;
-
-    my $pair = Algorithm::Pair::Best->new( ? options ? );
-
-    $pair->add( item, ? item, ... ? );
-
-    @pairList = $pair->pick( ? $window ? );
-
-=head1 DESCRIPTION
-
-After creating an Algorithm::Pair::Best-E<gt>B<new> object, B<add> a list of
-items (players) to be paired.  B<add> connects the new items into a linked
-list.  The linked list must consist of an even number of items or you'll get
-an error when you try to B<pick> the pairs.
-
-Pairings are determined partially by the original order items were added, but
-more importantly, items are paired based on scores which are determined by an
-B<info> hash used to attach any random data to the item, and user supplied
-functions to provide a score for each item in relation to other items.  It
-may be convenient to add access methods to the Algorithm::Pair::Best package
-from the main namespace (see the scoreSubs option to B<new> below for an
-example).
-
-Algorithm::Pair::Best-E<gt>B<pick> explores all combinations of items and
-returns the pairing with the best (highest) score.  This can be an expensive
-proposition - the number of combinations goes up very fast with respect to the
-number of items:
-
-    items combinations
-      2         1       (1)
-      4         3       (1 * 3)
-      6        15       (1 * 3 * 5)
-      8       105       (1 * 3 * 5 * 7)
-     10       945       (1 * 3 * 5 * 7 * 9
-     12     10395       (1 * 3 * 5 * 7 * 9 * 11)
-     14    135135       (1 * 3 * 5 * 7 * 9 * 11 * 13)
-
-It is clearly unreasonable to try to pair a significant number of items.  On
-my system it takes about 2 seconds to pair 12 items (6 pairs), and 20 seconds
-to pair 14 items (with no 'negative scores only' optimization).  Trying to
-completely pair even 30 items would take too long.
-
-Fortunately, there is a way to get pretty good results for large numbers, even
-if they're not perfect.  Instead of trying to pair the whole list at once,
-Algorithm::Pair::Best-E<gt>B<pick> pairs a series of smaller groups to get
-good 'local' results.  The B<new> method accepts a B<window> option to limit
-the number of pairs in each window.  The B<window> option can also be
-overridden by calling B<pick> with an explicit window argument:
-
-    $pair->pick($window);
-
-See the description of the B<window> option below.
-
-=cut
 
 # first, some boilerplate:
 use strict;
 require 5.001;
-BEGIN {
-    our $VERSION = sprintf "%d.%03d", '$Revision: 1.9 $' =~ /(\d+)/g;
-}
 
 
 # an Algorithm::Pair::Best is a pairing object.  it is one member of a linked list of
@@ -91,6 +27,27 @@ BEGIN {
 package Algorithm::Pair::Best;
 use Carp;
 
+our @ISA = qw(Exporter);
+
+# Items to export into callers namespace by default. Note: do not export
+# names by default without a very good reason. Use EXPORT_OK instead.
+# Do not simply export all your public functions/methods/constants.
+
+# This allows declaration       use PackageName ':all';
+# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
+# will save memory.
+our %EXPORT_TAGS = ( 'all' => [ qw(
+) ] );
+
+our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
+
+our @EXPORT = qw(
+);
+
+our $VERSION = '1.036'; # VERSION
+
+# ABSTRACT: deprecated - use Algorithm::Pair::Best2
+
 # uncomment the following line if perl -d is giving you recursion warnings:
 # $DB::deep = 1000;     # potentially deep recursion here
 
@@ -133,6 +90,342 @@ my %fields = (
 #
 #####################################################
 
+
+sub new {
+    my ($proto, %args) = @_;
+
+    my $my = { %fields };       # copy of default fields
+    $my->{info}    = {};        # user info
+    $my->{citems}  = [];        # candidate items, sorted by suitability
+    $my->{cscores} = {};        # cache of candidate scores
+    bless($my, ref($proto) || $proto);
+    # transfer user args
+    foreach my $a (keys(%args)) {
+        if ($a eq 'state') {
+            $my->state($args{$a});
+        } elsif ($a eq 'items') {
+            $my->items(@{$args{$a}});
+        } elsif ($a eq 'bestScore') {
+            $my->bestScore($args{$a});
+        } elsif ($a eq 'bestPairs') {
+            $my->bestPairs(@{$args{$a}});
+        } elsif ($a eq 'window') {
+            $my->window($args{$a});
+        } elsif ($a eq 'negOnly') {
+            $my->negOnly($args{$a});
+        } elsif ($a eq 'scoreSubs') {
+            $my->scoreSubs($args{$a});
+        } elsif (($a eq 'info') or
+                 ($a eq 'citems') or
+                 ($a eq 'cscores') or
+                 ($a eq 'opp') or
+                 ($a eq 'next')) {
+            $my->{$a} = $args{$a};
+        } else {
+            croak "$a is not a legal option to Algorithm::Pair::Best->new";
+        }
+    }
+    return $my;
+}
+
+
+# accessor methods for class variables
+sub state     {my $my = shift; ${$my->{state}}     = shift if (@_); return ${$my->{state}}    ;}
+sub items     {my $my = shift; @{$my->{items}}     = @_    if (@_); return @{$my->{items}}    ;}
+sub bestPairs {my $my = shift; @{$my->{bestPairs}} = @_    if (@_); return @{$my->{bestPairs}};}
+sub bestScore {my $my = shift; ${$my->{bestScore}} = shift if (@_); return ${$my->{bestScore}};}
+sub currScore {my $my = shift; ${$my->{currScore}} = shift if (@_); return ${$my->{currScore}};}
+sub window    {my $my = shift; ${$my->{window}}    = shift if (@_); return ${$my->{window}}   ;}
+sub negOnly   {my $my = shift; ${$my->{negOnly}}   = shift if (@_); return ${$my->{negOnly}}  ;}
+sub scoreSubs {my $my = shift; ${$my->{scoreSubs}} = shift if (@_); return ${$my->{scoreSubs}};}
+# accessor methods for object data
+sub info      {my $my = shift; $my->{info}      = shift if (@_); return $my->{info}     ;}
+sub citems    {my $my = shift; $my->{citems}    = shift if (@_); return $my->{citems}   ;}
+sub cscores   {my $my = shift; $my->{cscores}   = shift if (@_); return $my->{cscores}  ;}
+sub opp       {my $my = shift; $my->{opp}       = shift if (@_); return $my->{opp}      ;}
+sub next      {my $my = shift; $my->{next}      = shift if (@_); return $my->{next}     ;}
+
+
+sub add {
+    my $my = shift;
+
+    $my->state('add');
+    my @items;
+    foreach my $item (@_) {
+        my $pairItem = Algorithm::Pair::Best->new(info => $item);
+        if (exists($my->{lastItem})) {
+            $my->{lastItem}->next($pairItem);   # link new item into chain
+        }
+        $my->{lastItem} = $pairItem;            # set new last item
+        push(@items, $pairItem);
+        push(@{$my->{items}}, $pairItem);
+        $my->next($pairItem) unless (defined($my->next));
+    }
+    if (@items > 1) {
+        return wantarray ? @items : \@items;
+    }
+    return $items[0];
+}
+
+
+sub score {
+    my ($my, $candidate, $new_score) = @_;
+
+    if (defined($new_score)) {
+        $my->cscores->{$candidate} = $new_score; # cache it
+        $candidate->cscores->{$my} = $new_score; # cache his too
+    }
+    return($my->{cscores}{$candidate}) if (defined($my->{cscores}{$candidate}));
+    my $score = 0;
+    foreach (@{$my->scoreSubs}) {
+        $score += $my->$_($candidate);
+    }
+    $my->cscores->{$candidate} = $score; # cache it
+    $candidate->cscores->{$my} = $score; # cache his while we're at it
+    return $score;
+}
+
+
+
+sub sortCandidates {
+    my ($my) = @_;
+
+    foreach my $item (@{$my->{items}}) {
+        my @citems = sort( { $item->score($b) <=> $item->score($a) } @{$my->{items}});
+        $item->citems(\@citems);
+        foreach my $cs (values(%{$item->{cscore}})) {
+            ${$my->{negOnly}} = 0 if ($cs > 0);
+        }
+    }
+    $my->state('sort');
+}
+
+
+sub pick {
+    my ($my, $window) = @_;
+
+    $window = $my->window unless(defined($window)); # size of sliding window
+    my (@pairs, %sanity);
+    my $alreadyPaired = 0;
+    my $notPaired = 0;
+    while (@pairs < @{$my->{items}}) {
+        my $top = $my->wpick($window); # pick top pairs
+        my $save = 1;
+        if (((@{$top} + @pairs) >= @{$my->{items}}) or
+            (@{$top} < 2 * $window)) {
+            $save = @{$top} / 2;        # done - empty the list
+        }
+        while ($save--) {
+            my $p1 = shift @{$top};
+            my $p2 = shift @{$top};
+            $my->progress($p1, $p2) if (defined(&progress));
+            $p1->opp($p2);              # take these two out of contention
+            $p2->opp($p1);
+            push (@pairs, $p1, $p2);
+            $alreadyPaired++ if (exists($sanity{$p1}));
+            $alreadyPaired++ if (exists($sanity{$p2}));
+            $sanity{$p1} = $p2;         # yeah, I know we 'can't usefully use
+            $sanity{$p2} = $p1;         #       refs as hash keys', but we
+                                        #       don't need the ref back here,
+                                        #       just need a unique key
+        }
+    }
+    foreach (@{$my->{items}}) {
+        $notPaired++ unless (exists($sanity{$_}));
+        delete($sanity{$_});
+    }
+    my $msg = '';
+    $msg .= "$alreadyPaired ITEMS ALREADY PAIRED!\n" if($alreadyPaired);
+    $msg .= "$notPaired ITEMS NOT PAIRED!\n" if($notPaired);
+    $msg .= scalar(keys(%sanity)) . " ITEMS EXTRA!\n" if(scalar(keys(%sanity)));
+    croak "$msg" unless ($msg eq '');
+    return wantarray ? @pairs : \@pairs;
+}
+
+
+
+sub wpick {
+    my ($my, $window) = @_;
+
+    unless($my->state eq 'pick') {
+        $my->sortCandidates;
+    }
+    $my->state('pick');
+    my $avail = 0;
+    foreach my $p (@{$my->{items}}) {
+        next if (defined($p->opp));     # skip if already paired
+        $my = $p if($avail == 0);       # make my be first available
+        $avail++;
+        last if ($avail >= $window * 2);
+    }
+    if ($avail <= 1) {
+        if ($avail == 1) {
+            carp "Can't pick 1 - returning empty pairing list (last player will be left out)";
+        }
+        return ([]);
+    }
+    if ($avail % 2) {
+        $avail--;
+        carp "Can't pick from an odd number - last player will be left out";
+    }
+
+    $avail /= 2;
+    if ($avail < $window) {
+        carp "Not enough candidates: reducing maxPair to $avail";
+    }
+
+    $my->bestScore(-999999);
+    @{$my->{bestPairs}} = ();
+    $my->currScore(0);
+    @{$my->{currPairs}} = ();
+    # kick off recursive pairing
+    $my->_rpick($my->next, $avail, 0);        # starting candidate is my next, and depth is 0
+    if ($my->{oddError}) {      # shouldn't be possible
+        carp STDERR "Warning: odd number, last player not paired!\n"
+    }
+    return($my->{bestPairs});
+}
+
+
+sub _rpick {
+    my ($my, $firstCandidate, $window, $deep) = @_;
+
+    # number of candidates to try to pair with - any more
+    #    just eats time without improving the quality of the pairing
+    my $maxCand = (2 * $window) - 1;
+    push(@{$my->{currPairs}}, $my);             # put myself on the pairing list
+    my $cIdx = 0;
+    my $candidate;
+    for (my $ii = 0; $ii < $maxCand; $ii++) {
+        do {
+            $candidate = $my->citems->[$cIdx++];
+        } while (($cIdx <= scalar(@{$my->citems})) and
+                 (defined($candidate->opp) or   # find next unpaired in candidate list
+                  ($candidate == $my)));        # make sure it's not me!
+        last unless ($cIdx <= scalar(@{$my->citems}));
+        my $cscore = $my->score($candidate);
+        # call callers scoreFunc if defined
+        $cscore = $my->scoreFunc($candidate, $cscore) if(defined(&scoreFunc));
+        ${$my->{currScore}} += $cscore;         # add opp's score to total
+        if (${$my->{negOnly}} and (${$my->{currScore}} < ${$my->{bestScore}})) {
+            # we're below best, and it can only get worse, skip!
+        } else {
+            $candidate->opp($my);               # pair him with me
+            $my->opp($candidate);               # and me with him
+            push(@{$my->{currPairs}}, $candidate); # put this opp on the pairing list
+            if ($window > 1) {
+                my $next = $my;
+                while (defined($next) and defined($next->opp)) {
+                    $next = $next->next;
+                }
+                if (defined($next)) {
+                    $next->_rpick($firstCandidate, $window - 1, $deep + 1);
+                } else {
+                    carp "Out of candidates too soon!"; # shouldn't happen
+                }
+            } else {
+                # choose best of previous or current:
+                if (${$my->{currScore}} > ${$my->{bestScore}}) {
+                    ${$my->{bestScore}} = ${$my->{currScore}};
+                    @{$my->{bestPairs}} = ();
+                    @{$my->{bestPairs}} = @{$my->{currPairs}};
+                }
+            }
+            pop(@{$my->{currPairs}});           # remove opp from pairing list
+            $candidate->opp(undef);             # break pairing with this guy
+        }
+        ${$my->{currScore}} -= $cscore;         # remove opp's score to total
+    }
+    pop(@{$my->{currPairs}});                   # remove me from current list
+    $my->opp(undef);                            # I ain't got nobody...
+    return;
+}
+
+1;
+
+
+
+=pod
+
+=head1 NAME
+
+Algorithm::Pair::Best - deprecated - use Algorithm::Pair::Best2
+
+=head1 VERSION
+
+version 1.036
+
+=head1 SYNOPSIS
+
+    use Algorithm::Pair::Best;
+
+    my $pair = Algorithm::Pair::Best->new( ? options ? );
+
+    $pair->add( item, ? item, ... ? );
+
+    @pairList = $pair->pick( ? $window ? );
+
+=head1 DESCRIPTION
+
+Given a set of user-supplied scoring functions that compare all possible
+pairs of items, Algorithm::Pair::Best attempts to find the 'best'
+collective pairings of the entire group of items.
+
+After creating an Algorithm::Pair::Best-E<gt>B<new> object, B<add> a list of
+items (players) to be paired.  B<add> connects the new items into a linked
+list.  The total number of items B<add>ed to the linked list must consist
+of an even number of items or you'll get an error when you try to B<pick>
+the pairs.
+
+Pairings are determined partially by the original order items were added,
+but more importantly, items are paired based on scores which are determined
+by user supplied functions that provide a score for each item in relation
+to other items (see B<scoreSubs> below).  An B<info> hash is attached to
+each itme to assist the scoring functions.  It may be convenient to add
+access methods to the Algorithm::Pair::Best package from the main namespace
+(see the scoreSubs option to B<new> below for an example).
+
+Algorithm::Pair::Best-E<gt>B<pick> explores all combinations of items and
+returns the pairing with the best (highest) score.  This can be an expensive
+proposition - the number of combinations goes up very fast with respect to the
+number of items:
+
+    items combinations
+      2         1       (1)
+      4         3       (1 * 3)
+      6        15       (1 * 3 * 5)
+      8       105       (1 * 3 * 5 * 7)
+     10       945       (1 * 3 * 5 * 7 * 9
+     12     10395       (1 * 3 * 5 * 7 * 9 * 11)
+     14    135135       (1 * 3 * 5 * 7 * 9 * 11 * 13)
+
+It is clearly unreasonable to try to pair a significant number of items.  On
+my system it takes about 2 seconds to pair 12 items (6 pairs), and 20 seconds
+to pair 14 items (with no 'negative scores only' optimization).  Trying to
+completely pair even 30 items would take too long.
+
+Fortunately, there is a way to get pretty good results for large numbers, even
+if they're not perfect.  Instead of trying to pair the whole list at once,
+Algorithm::Pair::Best-E<gt>B<pick> pairs a series of smaller groups within
+a 'window' to get good 'local' results.  The list created by B<add> should
+be moderately sorted so that most reasonable candidates will be within
+window range of each other.
+
+The B<new> method accepts a B<window> option to limit the number of pairs
+in each window.  The B<window> option can also be overridden by calling
+B<pick> with an explicit window argument:
+
+    $pair->pick($window);
+
+See the description of the B<window> and B<pick> below.
+
+=head1 NAME
+
+Algorithm::Pair::Best is deprecated - use Algorithm::Pair::Best2
+
+Algorithm::Pair::Best - Perl module to select pairings (designed for Go
+tournaments, but can be used for anything, really).
+
 =head1 METHODS
 
 =over 4
@@ -146,11 +439,13 @@ paired.  It's just a control point for the collection of items to be paired.
 Items are added to the Algorithm::Pair::Best list with the <add> method (see
 below).
 
+=back
+
 =head2 Options
 
 =over 4
 
-=item B<window>
+=item B<window> => number of pairs
 
 Sets the default number of pairs in the sliding pairing window during a
 B<pick>.  Can also be set by passing a B<window> argument to B<pick>.
@@ -165,21 +460,20 @@ B<Games::Go::AGA> for a working example.
 
 Default: 5
 
-=item B<negOnly>
+=item B<negOnly> => true or false
 
 Enable/disable the 'negative scores only" optimization.  If any score greater
 than 0 is found during B<sortCandidates>, Algorithm::Pair::Best turns this
 flag off.
 
-IMPORTANT: If this flag is turned and a scoreSub can return a number greater
+IMPORTANT: If this flag is turned on and a scoreSub can return a number greater
 than 0, the resultant pairing may not be optimal, even locally.
 
 Default: 1 (enabled)
 
-=item B<scoreSubs>
+=item B<scoreSubs> => reference to array of scoring subroutines
 
-A reference to an array of scoring subroutines.  Scoring subroutines are
-called in array order as:
+Scoring subroutines are called in array order as:
 
     foreach my $s (@{$my->scoreSubs}) {
         $score += $my->$s($candidate);
@@ -194,7 +488,6 @@ The scoring subroutines should be symmetric so that:
 
     $a->$scoreSub($b) == $b->$scoreSub($a)
 
-
 Example:
 
 Note that the scores below are negative (Algorithm::Pair::Best searches for
@@ -258,46 +551,7 @@ Default: ref to empty array
 
 =back
 
-=cut
-
-sub new {
-    my ($proto, %args) = @_;
-
-    my $my = { %fields };       # copy of default fields
-    $my->{info}    = {};        # user info
-    $my->{citems}  = [];        # candidate items, sorted by suitability
-    $my->{cscores} = {};        # cache of candidate scores
-    bless($my, ref($proto) || $proto);
-    # transfer user args
-    foreach my $a (keys(%args)) {
-        if ($a eq 'state') {
-            $my->state($args{$a});
-        } elsif ($a eq 'items') {
-            $my->items(@{$args{$a}});
-        } elsif ($a eq 'bestScore') {
-            $my->bestScore($args{$a});
-        } elsif ($a eq 'bestPairs') {
-            $my->bestPairs(@{$args{$a}});
-        } elsif ($a eq 'window') {
-            $my->window($args{$a});
-        } elsif ($a eq 'negOnly') {
-            $my->negOnly($args{$a});
-        } elsif ($a eq 'scoreSubs') {
-            $my->scoreSubs($args{$a});
-        } elsif (($a eq 'info') or
-                 ($a eq 'citems') or
-                 ($a eq 'cscores') or
-                 ($a eq 'opp') or
-                 ($a eq 'next')) {
-            $my->{$a} = $args{$a};
-        } else {
-            croak "$a is not a legal option to Algorithm::Pair::Best->new";
-        }
-    }
-    return $my;
-}
-
-=item Accessor Methods
+=head2 Accessor Methods
 
 Accessor methods can read and write the following items:
 
@@ -328,23 +582,9 @@ Accessor methods can read and write the following items:
 Accessor methods set the appropriate variable if called with a parameter, and
 return the current (or new) value.
 
-=cut
+=head2 Other methods
 
-# accessor methods for class variables
-sub state     {my $my = shift; ${$my->{state}}     = shift if (@_); return ${$my->{state}}    ;}
-sub items     {my $my = shift; @{$my->{items}}     = @_    if (@_); return @{$my->{items}}    ;}
-sub bestPairs {my $my = shift; @{$my->{bestPairs}} = @_    if (@_); return @{$my->{bestPairs}};}
-sub bestScore {my $my = shift; ${$my->{bestScore}} = shift if (@_); return ${$my->{bestScore}};}
-sub currScore {my $my = shift; ${$my->{currScore}} = shift if (@_); return ${$my->{currScore}};}
-sub window    {my $my = shift; ${$my->{window}}    = shift if (@_); return ${$my->{window}}   ;}
-sub negOnly   {my $my = shift; ${$my->{negOnly}}   = shift if (@_); return ${$my->{negOnly}}  ;}
-sub scoreSubs {my $my = shift; ${$my->{scoreSubs}} = shift if (@_); return ${$my->{scoreSubs}};}
-# accessor methods for object data
-sub info      {my $my = shift; $my->{info}      = shift if (@_); return $my->{info}     ;}
-sub citems    {my $my = shift; $my->{citems}    = shift if (@_); return $my->{citems}   ;}
-sub cscores   {my $my = shift; $my->{cscores}   = shift if (@_); return $my->{cscores}  ;}
-sub opp       {my $my = shift; $my->{opp}       = shift if (@_); return $my->{opp}      ;}
-sub next      {my $my = shift; $my->{next}      = shift if (@_); return $my->{next}     ;}
+=over
 
 =item @pair_items = $pair-E<gt>B<add> ( item [ item ...] )
 
@@ -364,54 +604,21 @@ scalar context.
 Note: the returned pair_items list is not very useful since they have not yet
 been paired.
 
-=cut
-
-sub add {
-    my $my = shift;
-
-    $my->state('add');
-    my @items;
-    foreach my $item (@_) {
-        my $pairItem = Algorithm::Pair::Best->new(info => $item);
-        if (exists($my->{lastItem})) {
-            $my->{lastItem}->next($pairItem);   # link new item into chain
-        }
-        $my->{lastItem} = $pairItem;            # set new last item
-        push(@items, $pairItem);
-        push(@{$my->{items}}, $pairItem);
-        $my->next($pairItem) unless (defined($my->next));
-    }
-    if (@items > 1) {
-        return wantarray ? @items : \@items;
-    }
-    return $items[0];
-}
-
-=item $pair-E<gt>B<score> ( candidate )
+=item $pair-E<gt>B<score> ( candidate, ?new_score? )
 
 Returns the score (as calculated by calling the list of user-supplied
 scoreSubs) of the current pairing item relative to the candidate pairing item.
 
-=cut
-
-sub score {
-    my ($my, $candidate) = @_;
-
-    return($my->{cscores}{$candidate}) if (defined($my->{cscores}{$candidate}));
-    my $score = 0;
-    foreach (@{$my->scoreSubs}) {
-        $score += $my->$_($candidate);
-    }
-    $my->cscores->{$candidate} = $score; # cache it
-    $candidate->cscores->{$my} = $score; # cache his while we're at it
-    return $score;
-}
+The score is calculated only once, and the cached value is returned
+thereafter.
 
+If new_score is defined, the cached candidate and item scores are
+set to new_score.
 
 =item $pair-E<gt>B<sortCandidates>
 
-Sort each candidate list for each item.  This also caches the score for each
-candidate in each item.
+Sort each candidate list for each item.  This method calls B<score>
+(above) which caches the score for each candidate in each item.
 
 Normally this routine does not need to be called as the B<pick> method calls
 B<sortCandidate> before it starts picking.  However, if you would like to modify
@@ -419,7 +626,7 @@ candidate scores based on the sorting itself (for example, in the early rounds
 of a tournament, you may wish to avoid pairing the best matches against each
 other), you can call B<sortCandidates>, and then make scoring adjustments (use
 the B<citems> method to get a reference to the sorted list of candidates, then
-use $item-E<gt>B<score>($candidate, $newscore) to change the score).  After
+use $item-E<gt>B<score>($candidate, $new_score) to change the score).  After
 changing the score cache, calling the B<pick> method calls B<sortCandidates>
 once more which will re-sort based on the new scores cache.
 
@@ -429,21 +636,6 @@ optimize by skipping branches that already score below the current best
 pairing.  Any scores greater than 0 disable the 'negative scores only'
 (B<negOnly>) optimization.
 
-=cut
-
-sub sortCandidates {
-    my ($my) = @_;
-
-    foreach my $item (@{$my->{items}}) {
-        my @citems = sort( { $item->score($b) <=> $item->score($a) } @{$my->{items}});
-        $item->citems(\@citems);
-        foreach my $cs (values(%{$item->{cscore}})) {
-            ${$my->{negOnly}} = 0 if ($cs > 0);
-        }
-    }
-    $my->state('sort');
-}
-
 =item @pairs = $pair-E<gt>B<pick> ( ?$window? )
 
 Returns the best pairing found using the sliding window technique (calling
@@ -458,49 +650,6 @@ item[2] to item[3], etc.
 B<pick> performs a sanity check on the pairs list, checking that no item is
 paired twice, and that all items are paired.
 
-=cut
-
-sub pick {
-    my ($my, $window) = @_;
-
-    $window = $my->window unless(defined($window)); # size of sliding window
-    my (@pairs, %sanity);
-    my $alreadyPaired = 0;
-    my $notPaired = 0;
-    while (@pairs < @{$my->{items}}) {
-        my $top = $my->wpick($window); # pick top pairs
-        my $save = 1;
-        if (((@{$top} + @pairs) >= @{$my->{items}}) or
-            (@{$top} < 2 * $window)) {
-            $save = @{$top} / 2;        # done - empty the list
-        }
-        while ($save--) {
-            my $p1 = shift @{$top};
-            my $p2 = shift @{$top};
-            $my->progress($p1, $p2) if (defined(&progress));
-            $p1->opp($p2);              # take these two out of contention
-            $p2->opp($p1);
-            push (@pairs, $p1, $p2);
-            $alreadyPaired++ if (exists($sanity{$p1}));
-            $alreadyPaired++ if (exists($sanity{$p2}));
-            $sanity{$p1} = $p2;         # yeah, I know we 'can't usefully use
-            $sanity{$p2} = $p1;         #       refs as hash keys', but we
-                                        #       don't need the ref back here,
-                                        #       just need a unique key
-        }
-    }
-    foreach (@{$my->{items}}) {
-        $notPaired++ unless (exists($sanity{$_}));
-        delete($sanity{$_});
-    }
-    my $msg = '';
-    $msg .= "$alreadyPaired ITEMS ALREADY PAIRED!\n" if($alreadyPaired);
-    $msg .= "$notPaired ITEMS NOT PAIRED!\n" if($notPaired);
-    $msg .= scalar(keys(%sanity)) . " ITEMS EXTRA!\n" if(scalar(keys(%sanity)));
-    croak "$msg" unless ($msg eq '');
-    return wantarray ? @pairs : \@pairs;
-}
-
 =item $pair-E<gt>B<progress> ( $item0, $item1 )
 
 Each time a pair is finalized in the B<pick> routine above, it checks to see
@@ -526,8 +675,6 @@ finalized:
  # back to main:: namespace
  .  .  .
 
-=cut
-
 =item $pairsRef = $pair-E<gt>B<wpick> ( $window )
 
 Normally B<wpick> is only called by the B<pick> method.
@@ -544,57 +691,13 @@ last item.
 
 Note that while the pairing starts from the first item in the B<add> list, the
 returned pairs list may contain items from outside the first 2*B<$window> items
-in the B<add> list.  This is because each item has it's own ordered list of
+in the B<add> list.  This is because each item has its own ordered list of
 preferred pairs.  However, the first unpaired item in the B<add> list will be
 the first item in the returned list.
 
 Similarly, in the 'odd number of items remaining' situation, the discarded
 item is not neccessarily the last item in the B<add> list.
 
-=cut
-
-sub wpick {
-    my ($my, $window) = @_;
-
-    unless($my->state eq 'pick') {
-        $my->sortCandidates;
-    }
-    $my->state('pick');
-    my $avail = 0;
-    foreach my $p (@{$my->{items}}) {
-        next if (defined($p->opp));     # skip if already paired
-        $my = $p if($avail == 0);       # make my be first available
-        $avail++;
-        last if ($avail >= $window * 2);
-    }
-    if ($avail <= 1) {
-        if ($avail == 1) {
-            carp "Can't pick 1 - returning empty pairing list (last player will be left out)";
-        }
-        return ([]);
-    }
-    if ($avail % 2) {
-        $avail--;
-        carp "Can't pick from an odd number - last player will be left out";
-    }
-
-    $avail /= 2;
-    if ($avail < $window) {
-        carp "Not enough candidates: reducing maxPair to $avail";
-    }
-
-    $my->bestScore(-999999);
-    @{$my->{bestPairs}} = ();
-    $my->currScore(0);
-    @{$my->{currPairs}} = ();
-    # kick off recursive pairing
-    $my->_rpick($my->next, $avail, 0);        # starting candidate is my next, and depth is 0
-    if ($my->{oddError}) {      # shouldn't be possible
-        carp STDERR "Warning: odd number, last player not paired!\n"
-    }
-    return($my->{bestPairs});
-}
-
 =item $score = $pair-E<gt>B<scoreFunc> ( $candidate, $score )
 
 B<scoreFunc> is not defined in the Algorithm::Pair::Best package, but the
@@ -634,75 +737,15 @@ Example use of B<scoreFunc>:
  # back to main:: namespace
  .  .  .
 
-=cut
-
-sub _rpick {
-    my ($my, $firstCandidate, $window, $deep) = @_;
-
-    # number of candidates to try to pair with - any more
-    #    just eats time without improving the quality of the pairing
-    my $maxCand = (2 * $window) - 1;
-    push(@{$my->{currPairs}}, $my);             # put myself on the pairing list
-    my $cIdx = 0;
-    my $candidate;
-    for (my $ii = 0; $ii < $maxCand; $ii++) {
-        do {
-            $candidate = $my->citems->[$cIdx++];
-        } while (($cIdx <= scalar(@{$my->citems})) and
-                 (defined($candidate->opp) or   # find next unpaired in candidate list
-                  ($candidate == $my)));        # make sure it's not me!
-        last unless ($cIdx <= scalar(@{$my->citems}));
-        my $cscore = $my->score($candidate);
-        # call callers scoreFunc if defined
-        $cscore = $my->scoreFunc($candidate, $cscore) if(defined(&scoreFunc));
-        ${$my->{currScore}} += $cscore;         # add opp's score to total
-        if (${$my->{negOnly}} and (${$my->{currScore}} < ${$my->{bestScore}})) {
-            # we're below best, and it can only get worse, skip!
-        } else {
-            $candidate->opp($my);               # pair him with me
-            $my->opp($candidate);               # and me with him
-            push(@{$my->{currPairs}}, $candidate); # put this opp on the pairing list
-            if ($window > 1) {
-                my $next = $my;
-                while (defined($next) and defined($next->opp)) {
-                    $next = $next->next;
-                }
-                if (defined($next)) {
-                    $next->_rpick($firstCandidate, $window - 1, $deep + 1);
-                } else {
-                    carp "Out of candidates too soon!"; # shouldn't happen
-                }
-            } else {
-                # choose best of previous or current:
-                if (${$my->{currScore}} > ${$my->{bestScore}}) {
-                    ${$my->{bestScore}} = ${$my->{currScore}};
-                    @{$my->{bestPairs}} = ();
-                    @{$my->{bestPairs}} = @{$my->{currPairs}};
-                }
-            }
-            pop(@{$my->{currPairs}});           # remove opp from pairing list
-            $candidate->opp(undef);             # break pairing with this guy
-        }
-        ${$my->{currScore}} -= $cscore;         # remove opp's score to total
-    }
-    pop(@{$my->{currPairs}});                   # remove me from current list
-    $my->opp(undef);                            # I ain't got nobody...
-    return;
-}
-
-1;
-
-__END__
-
 =back
 
 =head1 SEE ALSO
 
-=over 0
+=over
 
-=item o gopair(1)
+=item gopair(1)
 
-The B<gopair> script from the Games::Go::AGA package uses
+The B<gopair> script from the Games::Go::GoPair package uses
 Algorithm::Pair::Best to run pairings for a go tournament
 
 =back
@@ -719,5 +762,20 @@ This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.5 or,
 at your option, any later version of Perl 5 you may have available.
 
+=head1 AUTHOR
+
+Reid Augustin <reid@hellosix.com>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2011 by Reid Augustin.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
 
+
+__END__
+
+
